package core.services;

import core.api.models.timemanager.tasks.repeatinginfo.DeadlineStrategy;
import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatDayInfo;
import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatWeekDayInfo;
import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatWeekInfo;
import core.entities.timemanager.*;
import core.repositories.timemanager.ScheduleRepository;
import core.repositories.timemanager.TaskRepository;
import core.repositories.timemanager.TaskSerieItemRepository;
import core.repositories.timemanager.TaskSeriesRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

@Service
public class TaskSeriesService {

    @Autowired private TaskRepository taskRepository;
    @Autowired private TaskSeriesRepository taskSeriesRepository;
    @Autowired private TaskSerieItemRepository taskSerieItemRepository;
    @Autowired private ScheduleRepository scheduleRepository;


    public ServiceExitCode createTaskSeries(TaskRepeatWeekInfo taskRepeatInfo) {
        List<Task> createdTasks = new ArrayList<>();
        TaskSerie taskSerie = new TaskSerie();
        List<AbstractSchedule> abstractSchedules = new ArrayList<>();

        for(TaskRepeatWeekDayInfo taskRepeatDayInfo : taskRepeatInfo.getWeekDayInfos()) {
            Optional<Task> task = taskRepository.findById(taskRepeatDayInfo.getTaskID());
            if(task.isEmpty()) return ServiceExitCode.MISSING_ENTITY;

            TaskSerieItem rootItem = taskSerie.addTask(task.get());
            task.get().setTaskSerieItem(rootItem);

            LocalDate currentTaskDate = task.get().getStartDate().plusDays(taskRepeatDayInfo.getOffset());
            while(currentTaskDate.isBefore(taskRepeatInfo.getEndDate())) {
                Task clonedTask = Task.cloneTask(task.get());
                clonedTask.setStartDate(currentTaskDate);

                TaskSerieItem taskSerieItem = taskSerie.addTask(clonedTask);
                clonedTask.setTaskSerieItem(taskSerieItem);
                createdTasks.add(clonedTask);

                abstractSchedules.addAll(cloneSchedules(task.get(), clonedTask));
                currentTaskDate = currentTaskDate.plusDays(taskRepeatDayInfo.getOffset());
            }
        }

        taskSerie.getTasks().sort(Comparator.comparing(o -> o.getTask().getStartDate()));
        for(int i=0; i<taskSerie.getTasks().size(); i++) {
            taskSerie.getTasks().get(i).setSeriesIndex(i+1);
            if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.DEADLINE_EQUAL_START) {
                taskSerie.getTasks().get(i).getTask().setDeadline(taskSerie.getTasks().get(i).getTask().getStartDate());
            } else {
                if(i + 1 == taskSerie.getTasks().size()) {
                    int firstWeekDayIndex = i % taskRepeatInfo.getWeekDayInfos().size();
                    LocalDate reference_start = taskSerie.getTasks().get(firstWeekDayIndex).getTask().getStartDate();
                    LocalDate reference_deadline = taskSerie.getTasks().get(firstWeekDayIndex).getTask().getDeadline();

                    Duration duration = Duration.between(reference_start, reference_deadline);
                    long days = duration.toDays();
                    taskSerie.getTasks().get(i).getTask().setDeadline(taskSerie.getTasks().get(i).getTask().getStartDate().plusDays(days));
                } else {
                    taskSerie.getTasks().get(i).getTask().setDeadline(taskSerie.getTasks().get(i+1).getTask().getStartDate().minusDays(1));
                }
            }
        }

        taskSeriesRepository.save(taskSerie);
        taskRepository.saveAll(createdTasks);
        taskSerieItemRepository.saveAll(taskSerie.getTasks());
        scheduleRepository.saveAll(abstractSchedules);

        return ServiceExitCode.OK;
    }

    public ServiceExitCode createTaskSeries(Task rootTask, TaskRepeatDayInfo taskRepeatInfo) {
        if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.FIX_DEADLINE) {
            return ServiceExitCode.INVALID_PARAMETER;
        }

        List<Task> taskList = new ArrayList<>();
        List<AbstractSchedule> abstractSchedules = new ArrayList<>();
        TaskSerie taskSerie = new TaskSerie();
        TaskSerieItem rootItem = taskSerie.addTask(rootTask);
        rootTask.setTaskSerieItem(rootItem);

        LocalDate currentTaskDate = rootTask.getStartDate().plusDays(taskRepeatInfo.getOffset());
        while(currentTaskDate.isBefore(taskRepeatInfo.getEndingDate())) {
            Task task = Task.cloneTask(rootTask);
            task.setStartDate(currentTaskDate);
            if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.DEADLINE_EQUAL_START) {
                task.setDeadline(currentTaskDate);
            } else if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.DEADLINE_FIT_START) {
                task.setDeadline(currentTaskDate.plusDays(taskRepeatInfo.getOffset()-1));
            }
            TaskSerieItem taskSerieItem = taskSerie.addTask(task);
            taskList.add(task);
            task.setTaskSerieItem(taskSerieItem);

            abstractSchedules.addAll(cloneSchedules(rootTask, task));
            currentTaskDate = currentTaskDate.plusDays(taskRepeatInfo.getOffset());
        }

        taskSeriesRepository.save(taskSerie);
        taskRepository.saveAll(taskList);
        taskSerieItemRepository.saveAll(taskSerie.getTasks());
        scheduleRepository.saveAll(abstractSchedules);

        return ServiceExitCode.OK;
    }

    public List<AbstractSchedule> cloneSchedules(Task previousTask, Task nextTask) {
        long numberDays = ChronoUnit.DAYS.between(previousTask.getStartDate(), nextTask.getStartDate());

        List<AbstractSchedule> clonedSchedules = new ArrayList<>();
        for(AbstractSchedule abstractSchedule : previousTask.getBasicTaskSchedules()) {
            AbstractSchedule clonedSchedule = abstractSchedule.cloneSchedule();
            clonedSchedule.shiftSchedule(numberDays);

            clonedSchedules.add(clonedSchedule);
        }
        return clonedSchedules;
    }

    public void deleteTaskSeriesItem(Task task) {
        TaskSerieItem item = task.getTaskSerieItem();
        TaskSerie taskSerie = task.getTaskSerieItem().getTaskSerie();
        taskSerie.getTasks().remove(item);
        task.setTaskSerieItem(null);
        taskSerieItemRepository.delete(item);
        if(taskSerie.getTasks().isEmpty()) {
            for(TaskSerieItem taskSerieItem : taskSerie.getTasks()) {
                taskSerieItem.setTaskSerie(null);
            }
            taskSerie.getTasks().clear();
            taskSeriesRepository.delete(taskSerie);
        } else {
            repearIndexing(taskSerie);
        }
    }

    private void repearIndexing(TaskSerie taskSerie) {
        taskSerie.getTasks().sort(Comparator.comparingInt(TaskSerieItem::getSeriesIndex));
        List<TaskSerieItem> updatedItems = new ArrayList<>();
        int currentIndex = 1;
        for(TaskSerieItem taskSerieItem : taskSerie.getTasks()) {
            if(taskSerieItem.getSeriesIndex() != currentIndex) {
                taskSerieItem.setSeriesIndex(currentIndex);
                updatedItems.add(taskSerieItem);
            }

            currentIndex++;
        }

        taskSerieItemRepository.saveAll(updatedItems);
    }

    public void deleteTaskSerieByTaskgroup(Taskgroup taskgroup) {
        taskSerieItemRepository.deleteByTaskgroup(taskgroup);
        taskSeriesRepository.deleteUnreferenced();
    }

    public static String convertIndexToString(int index) {
        if(index < 10) {
            return "0" + index;
        } else {
            return String.valueOf(index);
        }
    }

    public static String insertNameIndex(int seriesIndex, String taskName) {
        return taskName.replaceAll("\\$\\{i}", convertIndexToString(seriesIndex));
    }
}
