package core.services.ntfy;


import core.entities.User;
import core.entities.timemanager.AbstractSchedule;
import core.entities.timemanager.AdvancedTaskSchedule;
import core.repositories.timemanager.ScheduleRepository;
import core.repositories.timemanager.TaskRepository;
import core.services.TaskScheduleService;
import org.quartz.*;
import org.quartz.impl.StdSchedulerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;

@Service
public class TaskSchedulingService {

    @Value("${frontend.domain}")
    private String frontend_domain;

    @Autowired private TaskScheduleService taskScheduleService;

    private Scheduler scheduler;

    public void scheduleStartingTask(AdvancedTaskSchedule advancedTaskSchedule) throws SchedulerException {
        scheduler = StdSchedulerFactory.getDefaultScheduler();
        scheduler.start();

        JobDetail job = JobBuilder.newJob(NtfyTaskStartNotification.class)
                .withIdentity(String.valueOf(advancedTaskSchedule.getScheduleID()))
                .build();
        job.getJobDataMap().put("task", advancedTaskSchedule.getTask().getTaskName());
        job.getJobDataMap().put("ntfy_host", advancedTaskSchedule.getTask().getTaskgroup().getUser().getNtfy_host());
        job.getJobDataMap().put("ntfy_topic", advancedTaskSchedule.getTask().getTaskgroup().getUser().getNtfy_topic());
        job.getJobDataMap().put("frontend_domain", frontend_domain);

        Trigger trigger = TriggerBuilder.newTrigger().startAt(calculateDelayInMillis(advancedTaskSchedule.getScheduleStart())).build();

        scheduler.scheduleJob(job, trigger);
    }

    public void sendRunningTaskNotification(AbstractSchedule abstractSchedule) {
        HttpClient httpClient = HttpClient.newHttpClient();
        User user = abstractSchedule.getTask().getTaskgroup().getUser();
        try {
            HttpRequest request = HttpRequest.newBuilder()
                    .uri(new URI(user.getNtfy_host()+ "/" + user.getNtfy_topic()))
                    .POST(HttpRequest.BodyPublishers.ofString("Running Task " + abstractSchedule.getTask().getTaskName()))
                    .header("Tags", "heavy_check_mark")
                    .header("Title", "Task Running")
                    .header("Actions", "view, Open TimeScheduler, "+frontend_domain+", clear=true")
                    .build();

            httpClient.sendAsync(request, HttpResponse.BodyHandlers.ofString());
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private static Date calculateDelayInMillis(LocalDateTime executionTime) {
        return  Date.from(executionTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    @Scheduled(cron = "0 0 0 * * *")
    public void scheduleStartReminders() throws SchedulerException {
        List<AdvancedTaskSchedule> advancedTaskSchedules = taskScheduleService.findSchedulesByDate(LocalDate.now());
        for(AdvancedTaskSchedule advancedTaskSchedule : advancedTaskSchedules) {
            if(advancedTaskSchedule.getTask().getTaskgroup().getUser().getNtfy_host() != null) {
                scheduleStartingTask(advancedTaskSchedule);
            }
        }
    }

    public void stopStartReminderNotification(AdvancedTaskSchedule schedule) throws SchedulerException {
        scheduler.deleteJob(JobKey.jobKey(String.valueOf(schedule.getScheduleID())));
    }
}
