import {Component, OnInit, ViewChild} from '@angular/core';
import {NavigationLink, NavigationLinkListComponent} from "../../navigation-link-list/navigation-link-list.component";
import {
  AdvancedScheduleInfo,
  BasicScheduleEntityInfo, BasicScheduleInfo, ScheduleInfo, ScheduleService,
  TaskEntityInfo,
  TaskgroupEntityInfo,
  TaskgroupService,
  TaskService, TaskShortInfo
} from "../../../api";
import {ActivatedRoute, Router} from "@angular/router";
import {CalendarDateFormatter, CalendarEvent, CalendarEventTimesChangedEvent, CalendarView} from "angular-calendar";
import { Subject } from 'rxjs';
import {CalendarDatePipe} from "angular-calendar/modules/common/calendar-date/calendar-date.pipe";
import {BasicSchedulerComponent} from "../basic-scheduler/basic-scheduler.component";
import * as events from "events";
import { EventColor } from 'calendar-utils';
import {AdvancedSchedulerComponent} from "../advanced-scheduler/advanced-scheduler.component";
import * as moment from "moment";

const colors: Record<string, EventColor> = {
  red: {
    primary: '#ad2121',
    secondary: '#FAE3E3',
  },
  blue: {
    primary: '#1e90ff',
    secondary: '#D1E8FF',
  },
  yellow: {
    primary: '#e3bc08',
    secondary: '#FDF1BA',
  },
};
@Component({
  selector: 'app-scheduler',
  templateUrl: './scheduler.component.html',
  styleUrls: ['./scheduler.component.css']
})
export class SchedulerComponent implements OnInit{
  defaultNavigationLinkPath: NavigationLink[] = [
    {
      linkText: "Dashboard",
      routerLink: ['/']
    },
    {
      linkText: "Taskgroups",
      routerLink: ["/taskgroups"]
    }
  ]
  taskgroups: TaskgroupEntityInfo[] = []
  taskgroup: TaskgroupEntityInfo | undefined
  taskgroupPath: TaskgroupEntityInfo[] = []
  taskgroupID: number = -1;
  scheduleID: number = -1;
  editedSchedule: ScheduleInfo | undefined
  @ViewChild('navLinkList') navLinkListComponent: NavigationLinkListComponent | undefined

  task: TaskEntityInfo | undefined
  scheduleStrategy: number = 1


  /**************************************************/
  //Calendar-Stuff
  /**************************************************/
  view: CalendarView = CalendarView.Week;
  viewDate = new Date();
  daysInWeek = 7;
  refresh: Subject<void> = new Subject<void>()
  events: CalendarEvent[] = []

  @ViewChild('basicScheduler') basicScheduler: BasicSchedulerComponent | undefined
  @ViewChild('advancedScheduler') advancedScheduler: AdvancedSchedulerComponent | undefined

  constructor(private activatedRoute: ActivatedRoute,
              private taskgroupService: TaskgroupService,
              private taskService: TaskService,
              private scheduleService: ScheduleService,
              private router: Router) {
  }

  ngOnInit(): void {
    this.activatedRoute.paramMap.subscribe(params => {
      if (params.has('taskgroupID')) {
        this.taskgroupID = Number(params.get('taskgroupID'));
        this.taskgroupService.taskgroupsTaskgroupIDGet(this.taskgroupID).subscribe({
          next: resp => {
            this.taskgroups = resp.children
            this.taskgroupPath = resp.ancestors
            this.taskgroup = resp.taskgroupInfo;
            this.initializeNavigationLinkList()
          }
        })
      }

      if(params.has('taskID')) {
        this.taskService.tasksTaskIDGet(Number(params.get('taskID'))).subscribe({
          next: resp => {
            this.task = resp;
            this.initializeNavigationLinkList()
            this.fetschSchedules();
          }
        })
      }

      if(params.has('scheduleID')) {
        this.scheduleService.schedulesScheduleIDDetailsGet(parseInt(params.get('scheduleID')!)).subscribe({
          next: resp => {
            this.editedSchedule = resp;
            this.scheduleID = resp.scheduleID;
            if(resp.scheduleType == 'ADVANCED') {
              this.scheduleStrategy = 3;
            }
          }
        })
      }
    });
  }

  initializeNavigationLinkList() {
    if(this.taskgroup != undefined && this.task != undefined) {
      this.navLinkListComponent!.addNavigationLink(this.taskgroup!.taskgroupName, ['/taskgroups', this.taskgroup!.taskgroupID.toString()])
      this.taskgroupPath.forEach(taskgroupEntity => {
        this.navLinkListComponent!.addNavigationLink(taskgroupEntity.taskgroupName, ['/taskgroups', taskgroupEntity.taskgroupID.toString()]);
      })
      this.navLinkListComponent!.addNavigationLink(this.task!.taskName, ['/taskgroups', this.taskgroup!.taskgroupID.toString(), 'tasks', this.task.taskID.toString()]);
      this.navLinkListComponent!.addNavigationLink('Schedule', [this.taskgroup!.taskgroupID.toString(), 'tasks', this.task.taskID.toString(), 'schedule'])
    }
  }

  protected readonly CalendarView = CalendarView;

  timeClick(clickedDate: Date) {

    console.log(clickedDate)
    if(this.basicScheduler != undefined && this.scheduleStrategy === 1) {
      this.basicScheduler.setDate(clickedDate)
    } else if(this.advancedScheduler != undefined && this.scheduleStrategy === 3) {
      this.advancedScheduler.setDate(clickedDate)
      const endDate: Date = moment(clickedDate).add(30, 'm').toDate();
      this.advancedScheduler.setDate(endDate);
      this.events.push({
        title: this.computeTaskPath(this.taskgroupPath, this.task!),
        start: clickedDate,
        end: endDate,
        color: colors['yellow'],
        resizable: {
          beforeStart: true,
          afterEnd: true
        }
      })
      this.refresh.next();
    }
  }

  eventTimesChanged({
                      event,
                      newStart,
                      newEnd,
                    }: CalendarEventTimesChangedEvent): void {
    event.start = newStart;
    event.end = newEnd;
    this.refresh.next();

    this.advancedScheduler!.setStartTime(event.start);
    this.advancedScheduler!.setStopTime(event.end!);
  }

  onBasicSchedule(scheduleInfo: BasicScheduleEntityInfo) {
    this.events.push({
      start: new Date(scheduleInfo.scheduleDate),
      title: this.task!.taskName,
      color: colors['red'],
      allDay: true,
    })
    this.refresh.next();
  }

  fetschSchedules() {
    this.scheduleService.schedulesGet().subscribe({
      next: resp => {
        resp.forEach(abstractSchedule => {
          if(abstractSchedule.scheduleType == 'BASIC') {
            this.handleFetchedBasicSchedule(abstractSchedule as BasicScheduleInfo)
          } else if(abstractSchedule.scheduleType == 'ADVANCED') {
            this.handleFetchedAdvancedSchedule(abstractSchedule as AdvancedScheduleInfo)
          }

        })
      }
    })
  }

  handleFetchedBasicSchedule(schedule: BasicScheduleInfo) {
    let color: EventColor = colors['red']
    if(schedule.scheduleID === this.scheduleID) {
      color = colors['yellow']
    }

    if(schedule.scheduleType === 'BASIC') {
      this.events.push({
        start: new Date(schedule.scheduleDate),
        title: this.computeTaskPath(schedule.taskgroupPath, schedule.task),
        color: color,
        allDay: true,
        meta: {
          ID: this.scheduleID,
          taskgroupID: schedule.taskgroupPath[schedule.taskgroupPath.length-1].taskgroupID,
          taskID: schedule.task.taskID
        }
      })
      this.refresh.next();
    }
  }

  eventClicked({ event }: { event: CalendarEvent }): void {
    console.log(event.meta.taskgroupID);
    this.router.navigateByUrl("/taskgroups/" + event.meta.taskgroupID + "/tasks/" + event.meta.taskID )
  }

  computeTaskPath(taskgroupPath: TaskgroupEntityInfo[], task: TaskShortInfo | TaskEntityInfo) {
    let result = "";
    taskgroupPath.forEach(taskgroupPathPart => {
      result += taskgroupPathPart.taskgroupName + "/"
    });
    result += task!.taskName
    return result;
  }

  findTaskByTaskPathString(taskgroupPathString: string) :TaskShortInfo {
    return this.task!;
  }

  findEventByTaskPath(taskPath: string, advanced: boolean, scheduleID: number | undefined) {
    const events : CalendarEvent<any>[] = this.events.filter(event => event.title == taskPath);
    if(scheduleID != -1) {
      return this.events.find(calendarEvent => calendarEvent.meta.ID == scheduleID);
    }
    if(advanced) {
      return events.find(calendarEvent => !calendarEvent.allDay)
    } else {
      return events.find(calendarEvent => calendarEvent.allDay)
    }
  }

  eventTimesExternalChange(selectedDate: Date, start: boolean) {
    const event: CalendarEvent<any>| undefined = this.findEventByTaskPath(this.computeTaskPath(this.taskgroupPath, this.task!), true, this.scheduleID);
    if(event == undefined) {
      if(start) {
        this.events.push({
          title: this.computeTaskPath(this.taskgroupPath, this.task!),
          start: selectedDate,
          color: colors['yellow'],
          resizable: {
            beforeStart: true,
            afterEnd: true
          },
          meta: {
            ID: -1
          }
        })
      } else {
        const updatedDate = moment(selectedDate).subtract(30, 'm').toDate();
        this.events.push({
          title: this.computeTaskPath(this.taskgroupPath, this.task!),
          start: updatedDate,
          color: colors['yellow'],
          resizable: {
            beforeStart: true,
            afterEnd: true
          },
          meta: {
            ID: -1
          }
        })
        this.advancedScheduler!.setStartTime(updatedDate);
      }

    } else {
      if(start) {
        event.start = selectedDate;
      } else {
        event.end = selectedDate
      }
    }

    this.refresh.next();

  }

  onAdvancedSchedule(schedule: AdvancedScheduleInfo) {
    const event: CalendarEvent<any>| undefined = this.findEventByTaskPath(this.computeTaskPath(schedule.taskgroupPath, schedule.task), true, this.scheduleID);
    if(event == undefined) {
      this.events.push({
        title: this.computeTaskPath(schedule.taskgroupPath, schedule.task),
        start: new Date(schedule.scheduleStartTime),
        end: new Date(schedule.scheduleStopTime),
        color: colors['red'],
        resizable: {
          beforeStart: false,
          afterEnd: false
        },
        meta: {
          ID: this.scheduleID
        }
      })

      this.refresh.next();
    } else {
      event.color = colors['red'];
      event.resizable = {
        beforeStart: false,
        afterEnd: false
      }
      this.refresh.next();
    }

  }

  private handleFetchedAdvancedSchedule(schedule: AdvancedScheduleInfo) {
    let color: EventColor = colors['red']
    if(schedule.scheduleID === this.scheduleID) {
      color = colors['yellow']
    }

    this.events.push({
      start: new Date(schedule.scheduleStartTime),
      end: new Date(schedule.scheduleStopTime),
      title: this.computeTaskPath(schedule.taskgroupPath, schedule.task),
      color: color,
      allDay: false,
      meta: {
        ID: schedule.scheduleID
      }
    })
    this.refresh.next();
  }
}
